==> fabric-recipe-api-v1-8.1.28%2B65ffd0401e-sources.jar: net/fabricmc/fabric/impl/recipe/ingredient/CustomIngredientImpl.java <==
/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.impl.recipe.ingredient;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Stream;

import com.mojang.serialization.Codec;
import com.mojang.serialization.DataResult;
import org.jetbrains.annotations.Nullable;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredient;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredientSerializer;
import net.minecraft.class_10302;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1856;
import net.minecraft.class_2960;
import net.minecraft.class_6880;
import net.minecraft.class_6885;

/**
 * To test this API beyond the unit tests, please refer to the recipe provider in the datagen API testmod.
 * It contains various interesting recipes to test, and explains how to package them in a datapack.
 */
public class CustomIngredientImpl extends class_1856 {
	// Static helpers used by the API

	public static final String TYPE_KEY = "fabric:type";

	static final Map<class_2960, CustomIngredientSerializer<?>> REGISTERED_SERIALIZERS = new ConcurrentHashMap<>();

	public static final Codec<CustomIngredientSerializer<?>> field_46095 = class_2960.field_25139.flatXmap(identifier ->
					Optional.ofNullable(REGISTERED_SERIALIZERS.get(identifier))
							.map(DataResult::success)
							.orElseGet(() -> DataResult.error(() -> "Unknown custom ingredient serializer: " + identifier)),
			serializer -> DataResult.success(serializer.getIdentifier())
	);

	public static void registerSerializer(CustomIngredientSerializer<?> serializer) {
		Objects.requireNonNull(serializer.getIdentifier(), "CustomIngredientSerializer identifier may not be null.");

		if (REGISTERED_SERIALIZERS.putIfAbsent(serializer.getIdentifier(), serializer) != null) {
			throw new IllegalArgumentException("CustomIngredientSerializer with identifier " + serializer.getIdentifier() + " already registered.");
		}
	}

	@Nullable
	public static CustomIngredientSerializer<?> getSerializer(class_2960 identifier) {
		Objects.requireNonNull(identifier, "Identifier may not be null.");

		return REGISTERED_SERIALIZERS.get(identifier);
	}

	// Actual custom ingredient logic

	private final CustomIngredient customIngredient;
	@Nullable
	private List<class_6880<class_1792>> customMatchingItems;

	public CustomIngredientImpl(CustomIngredient customIngredient) {
		// We must pass a registry entry list that contains something that isn't air. It doesn't actually get used.
		super(class_6885.method_40246(class_1802.field_20391.method_40131()));

		this.customIngredient = customIngredient;
	}

	public List<class_6880<class_1792>> getCustomMatchingItems() {
		if (customMatchingItems == null) {
			customMatchingItems = customIngredient.getMatchingItems().toList();
		}

		return customMatchingItems;
	}

	@Override
	public CustomIngredient getCustomIngredient() {
		return customIngredient;
	}

	@Override
	public boolean requiresTesting() {
		return customIngredient.requiresTesting();
	}

	@Override
	public Stream<class_6880<class_1792>> method_8105() {
		return getCustomMatchingItems().stream();
	}

	@Override
	public boolean method_65799() {
		return getCustomMatchingItems().isEmpty();
	}

	@Override
	public boolean method_8093(class_1799 stack) {
		return customIngredient.test(stack);
	}

	@Override
	public boolean method_65798(class_6880<class_1792> registryEntry) {
		return getCustomMatchingItems().contains(registryEntry);
	}

	@Override
	public class_10302 method_64673() {
		return customIngredient.toDisplay();
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (!(o instanceof CustomIngredientImpl that)) return false;
		return customIngredient.equals(that.customIngredient);
	}

	@Override
	public int hashCode() {
		return customIngredient.hashCode();
	}
}

==> fabric-recipe-api-v1-8.1.29%2B65ffd0401e-sources.jar: net/fabricmc/fabric/impl/recipe/ingredient/CustomIngredientImpl.java <==
/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.impl.recipe.ingredient;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Stream;

import com.mojang.serialization.Codec;
import com.mojang.serialization.DataResult;
import org.jetbrains.annotations.Nullable;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredient;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredientSerializer;
import net.minecraft.class_10302;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1856;
import net.minecraft.class_2960;
import net.minecraft.class_6880;
import net.minecraft.class_6885;

/**
 * To test this API beyond the unit tests, please refer to the recipe provider in the datagen API testmod.
 * It contains various interesting recipes to test, and explains how to package them in a datapack.
 */
public class CustomIngredientImpl extends class_1856 {
	// Static helpers used by the API

	public static final String TYPE_KEY = "fabric:type";

	static final Map<class_2960, CustomIngredientSerializer<?>> REGISTERED_SERIALIZERS = new ConcurrentHashMap<>();

	public static final Codec<CustomIngredientSerializer<?>> CODEC = class_2960.field_25139.flatXmap(identifier ->
					Optional.ofNullable(REGISTERED_SERIALIZERS.get(identifier))
							.map(DataResult::success)
							.orElseGet(() -> DataResult.error(() -> "Unknown custom ingredient serializer: " + identifier)),
			serializer -> DataResult.success(serializer.getIdentifier())
	);

	public static void registerSerializer(CustomIngredientSerializer<?> serializer) {
		Objects.requireNonNull(serializer.getIdentifier(), "CustomIngredientSerializer identifier may not be null.");

		if (REGISTERED_SERIALIZERS.putIfAbsent(serializer.getIdentifier(), serializer) != null) {
			throw new IllegalArgumentException("CustomIngredientSerializer with identifier " + serializer.getIdentifier() + " already registered.");
		}
	}

	@Nullable
	public static CustomIngredientSerializer<?> getSerializer(class_2960 identifier) {
		Objects.requireNonNull(identifier, "Identifier may not be null.");

		return REGISTERED_SERIALIZERS.get(identifier);
	}

	// Actual custom ingredient logic

	private final CustomIngredient customIngredient;
	@Nullable
	private List<class_6880<class_1792>> customMatchingItems;

	public CustomIngredientImpl(CustomIngredient customIngredient) {
		// We must pass a registry entry list that contains something that isn't air. It doesn't actually get used.
		super(class_6885.method_40246(class_1802.field_20391.method_40131()));

		this.customIngredient = customIngredient;
	}

	public List<class_6880<class_1792>> getCustomMatchingItems() {
		if (customMatchingItems == null) {
			customMatchingItems = customIngredient.getMatchingItems().toList();
		}

		return customMatchingItems;
	}

	@Override
	public CustomIngredient getCustomIngredient() {
		return customIngredient;
	}

	@Override
	public boolean requiresTesting() {
		return customIngredient.requiresTesting();
	}

	@Override
	public Stream<class_6880<class_1792>> method_8105() {
		return getCustomMatchingItems().stream();
	}

	@Override
	public boolean method_65799() {
		return getCustomMatchingItems().isEmpty();
	}

	@Override
	public boolean method_8093(class_1799 stack) {
		return customIngredient.test(stack);
	}

	@Override
	public boolean method_65798(class_6880<class_1792> registryEntry) {
		return getCustomMatchingItems().contains(registryEntry);
	}

	@Override
	public class_10302 method_64673() {
		return customIngredient.toDisplay();
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (!(o instanceof CustomIngredientImpl that)) return false;
		return customIngredient.equals(that.customIngredient);
	}

	@Override
	public int hashCode() {
		return customIngredient.hashCode();
	}
}
